@@ -240,12 +240,6 @@ class Build:
     def __repr__(self):
         return "<Build %s>" % (self.builder.name,)
 
-    def __getstate__(self):
-        d = self.__dict__.copy()
-        if d.has_key('remote'):
-            del d['remote']
-        return d
-
     def blamelist(self):
         blamelist = []
         for c in self.allChanges():
@@ -291,6 +285,10 @@ class Build:
         for rq in self.requests:
             props.updateFromProperties(rq.properties)
 
+        # and finally, from the SourceStamp, which has properties via Change
+        for change in self.source.changes:
+            props.updateFromProperties(change.properties)
+
         # now set some properties of our own, corresponding to the
         # build itself
         props.setProperty("buildername", self.builder.name, "Build")
@@ -392,7 +390,7 @@ class Build:
         # add a count to the loser until it is unique.
         self.steps = []
         self.stepStatuses = {}
-        stepnames = []
+        stepnames = {}
         sps = []
 
         for factory, args in self.stepFactories:
@@ -407,17 +405,14 @@ class Build:
             step.setBuildSlave(self.slavebuilder.slave)
             step.setDefaultWorkdir(self.workdir)
             name = step.name
-            count = 1
-            while name in stepnames and count < 1000:
+            if stepnames.has_key(name):
+                count = stepnames[name]
                 count += 1
+                stepnames[name] = count
                 name = step.name + "_%d" % count
-            if count == 1000:
-                raise RuntimeError("reached 1000 steps with base name" + \
-                                   "%s, bailing" % step.name)
-            elif name in stepnames:
-                raise RuntimeError("duplicate step '%s'" % step.name)
+            else:
+                stepnames[name] = 0
             step.name = name
-            stepnames.append(name)
             self.steps.append(step)
 
             # tell the BuildStatus about the step. This will create a
