@@ -19,7 +19,7 @@ from buildbot.process.properties import Properties
 
 class BaseScheduler(service.MultiService, util.ComparableMixin):
     """
-    A Schduler creates BuildSets and submits them to the BuildMaster.
+    A Scheduler creates BuildSets and submits them to the BuildMaster.
 
     @ivar name: name of the scheduler
 
@@ -81,7 +81,7 @@ class BaseUpstreamScheduler(BaseScheduler):
 class Scheduler(BaseUpstreamScheduler):
     """The default Scheduler class will run a build after some period of time
     called the C{treeStableTimer}, on a given set of Builders. It only pays
-    attention to a single branch. You you can provide a C{fileIsImportant}
+    attention to a single branch. You can provide a C{fileIsImportant}
     function which will evaluate each Change to decide whether or not it
     should trigger a new build.
     """
@@ -95,11 +95,11 @@ class Scheduler(BaseUpstreamScheduler):
         """
         @param name: the name of this Scheduler
         @param branch: The branch name that the Scheduler should pay
-                       attention to. Any Change that is not on this branch
+                       attention to. Any Change that is not in this branch
                        will be ignored. It can be set to None to only pay
                        attention to the default branch.
         @param treeStableTimer: the duration, in seconds, for which the tree
-                                must remain unchanged before a build will be
+                                must remain unchanged before a build is
                                 triggered. This is intended to avoid builds
                                 of partially-committed fixes.
         @param builderNames: a list of Builder names. When this Scheduler
@@ -134,7 +134,7 @@ class Scheduler(BaseUpstreamScheduler):
             self.fileIsImportant = fileIsImportant
 
         self.importantChanges = []
-        self.unimportantChanges = []
+        self.allChanges = []
         self.nextBuildTime = None
         self.timer = None
         self.categories = categories
@@ -164,13 +164,14 @@ class Scheduler(BaseUpstreamScheduler):
     def addImportantChange(self, change):
         log.msg("%s: change is important, adding %s" % (self, change))
         self.importantChanges.append(change)
+        self.allChanges.append(change)
         self.nextBuildTime = max(self.nextBuildTime,
                                  change.when + self.treeStableTimer)
         self.setTimer(self.nextBuildTime)
 
     def addUnimportantChange(self, change):
         log.msg("%s: change is not important, adding %s" % (self, change))
-        self.unimportantChanges.append(change)
+        self.allChanges.append(change)
 
     def setTimer(self, when):
         log.msg("%s: setting timer to %s" %
@@ -191,9 +192,9 @@ class Scheduler(BaseUpstreamScheduler):
         # clear out our state
         self.timer = None
         self.nextBuildTime = None
-        changes = self.importantChanges + self.unimportantChanges
+        changes = self.allChanges
         self.importantChanges = []
-        self.unimportantChanges = []
+        self.allChanges = []
 
         # create a BuildSet, submit it to the BuildMaster
         bs = buildset.BuildSet(self.builderNames,
@@ -215,21 +216,21 @@ class AnyBranchScheduler(BaseUpstreamScheduler):
     fileIsImportant = None
 
     compare_attrs = ('name', 'branches', 'treeStableTimer', 'builderNames',
-                     'fileIsImportant', 'properties')
+                     'fileIsImportant', 'properties', 'categories')
 
     def __init__(self, name, branches, treeStableTimer, builderNames,
-                 fileIsImportant=None, properties={}):
+                 fileIsImportant=None, properties={}, categories=None):
         """
         @param name: the name of this Scheduler
         @param branches: The branch names that the Scheduler should pay
-                         attention to. Any Change that is not on one of these
+                         attention to. Any Change that is not in one of these
                          branches will be ignored. It can be set to None to
                          accept changes from any branch. Don't use [] (an
                          empty list), because that means we don't pay
                          attention to *any* branches, so we'll never build
                          anything.
         @param treeStableTimer: the duration, in seconds, for which the tree
-                                must remain unchanged before a build will be
+                                must remain unchanged before a build is
                                 triggered. This is intended to avoid builds
                                 of partially-committed fixes.
         @param builderNames: a list of Builder names. When this Scheduler
@@ -246,6 +247,7 @@ class AnyBranchScheduler(BaseUpstreamScheduler):
 
         @param properties: properties to apply to all builds started from this 
                            scheduler
+        @param categories: A list of categories of changes to accept 
         """
 
         BaseUpstreamScheduler.__init__(self, name, properties)
@@ -266,6 +268,7 @@ class AnyBranchScheduler(BaseUpstreamScheduler):
             assert callable(fileIsImportant)
             self.fileIsImportant = fileIsImportant
         self.schedulers = {} # one per branch
+        self.categories = categories
 
     def __repr__(self):
         return "<AnyBranchScheduler '%s'>" % self.name
@@ -290,6 +293,9 @@ class AnyBranchScheduler(BaseUpstreamScheduler):
         if self.branches is not None and branch not in self.branches:
             log.msg("%s ignoring off-branch %s" % (self, change))
             return
+        if self.categories is not None and change.category not in self.categories:
+            log.msg("%s ignoring non-matching categories %s" % (self, change))
+            return
         s = self.schedulers.get(branch)
         if not s:
             if branch:
@@ -487,7 +493,7 @@ class Nightly(BaseUpstreamScheduler):
                        % name)
 
         self.importantChanges   = [] 
-        self.unimportantChanges = [] 
+        self.allChanges = [] 
         self.fileIsImportant    = None 
         if fileIsImportant: 
             assert callable(fileIsImportant) 
@@ -584,7 +590,7 @@ class Nightly(BaseUpstreamScheduler):
 
         if  self.onlyIfChanged:
             if len(self.importantChanges) > 0: 
-                changes = self.importantChanges + self.unimportantChanges 
+                changes = self.allChanges 
                 # And trigger a build 
                 log.msg("Nightly Scheduler <%s>: triggering build" % self.name) 
                 bs = buildset.BuildSet(self.builderNames,
@@ -594,7 +600,7 @@ class Nightly(BaseUpstreamScheduler):
                 self.submitBuildSet(bs)
                 # Reset the change lists 
                 self.importantChanges = [] 
-                self.unimportantChanges = []
+                self.allChanges = []
             else: 
                 log.msg("Nightly Scheduler <%s>: skipping build - No important change" % self.name)
         else:
@@ -622,11 +628,12 @@ class Nightly(BaseUpstreamScheduler):
  
     def addImportantChange(self, change):
         log.msg("Nightly Scheduler <%s>: change %s from %s is important, adding it" % (self.name, change.revision, change.who)) 
+        self.allChanges.append(change) 
         self.importantChanges.append(change) 
  
     def addUnimportantChange(self, change):
         log.msg("Nightly Scheduler <%s>: change %s from %s is not important, adding it" % (self.name, change.revision, change.who)) 
-        self.unimportantChanges.append(change) 
+        self.allChanges.append(change) 
 
 
 class TryBase(BaseScheduler):
