@@ -3,14 +3,26 @@
 from zope.interface import implements
 from twisted.python import log
 from twisted.persisted import styles
-from twisted.internet import reactor, defer
+from twisted.internet import reactor, defer, threads
 from twisted.protocols import basic
 from buildbot.process.properties import Properties
 
+import weakref
 import os, shutil, sys, re, urllib, itertools
+import gc
 from cPickle import load, dump
 from cStringIO import StringIO
 
+try: # bz2 is not available on py23
+    from bz2 import BZ2File
+except ImportError:
+    BZ2File = None
+
+try:
+    from gzip import GzipFile
+except ImportError:
+    GzipFile = None
+
 # sibling imports
 from buildbot import interfaces, util, sourcestamp
 
@@ -184,6 +196,22 @@ class LogFileProducer:
             self.consumer.finish()
             self.consumer = None
 
+def _tryremove(filename, timeout, retries):
+    """Try to remove a file, and if failed, try again in timeout.
+    Increases the timeout by a factor of 4, and only keeps trying for
+    another retries-amount of times.
+
+    """
+    try:
+        os.unlink(filename)
+    except OSError:
+        if retries > 0:
+            reactor.callLater(timeout, _tryremove, filename, timeout * 4, 
+                              retries - 1)
+        else:
+            log.msg("giving up on removing %s after over %d seconds" %
+                    (filename, timeout))
+
 class LogFile:
     """A LogFile keeps all of its contents on disk, in a non-pickle format to
     which new entries can easily be appended. The file on disk has a name
@@ -201,13 +229,21 @@ class LogFile:
 
     finished = False
     length = 0
+    nonHeaderLength = 0
+    tailLength = 0
     chunkSize = 10*1000
     runLength = 0
+    # No max size by default
+    logMaxSize = None
+    # Don't keep a tail buffer by default
+    logMaxTailSize = None
+    maxLengthExceeded = False
     runEntries = [] # provided so old pickled builds will getChunks() ok
     entries = None
     BUFFERSIZE = 2048
     filename = None # relative to the Builder's basedir
     openfile = None
+    compressMethod = "bz2"
 
     def __init__(self, parent, name, logfilename):
         """
@@ -228,16 +264,22 @@ class LogFile:
             # is out of date, and we're overlapping with earlier builds now.
             # Warn about it, but then overwrite the old pickle file
             log.msg("Warning: Overwriting old serialized Build at %s" % fn)
+        dirname = os.path.dirname(fn)
+        if not os.path.exists(dirname):
+            os.makedirs(dirname)
         self.openfile = open(fn, "w+")
         self.runEntries = []
         self.watchers = []
         self.finishedWatchers = []
+        self.tailBuffer = []
 
     def getFilename(self):
         return os.path.join(self.step.build.builder.basedir, self.filename)
 
     def hasContents(self):
-        return os.path.exists(self.getFilename())
+        return os.path.exists(self.getFilename() + '.bz2') or \
+            os.path.exists(self.getFilename() + '.gz') or \
+            os.path.exists(self.getFilename())
 
     def getName(self):
         return self.name
@@ -261,6 +303,17 @@ class LogFile:
             # don't close it!
             return self.openfile
         # otherwise they get their own read-only handle
+        # try a compressed log first
+        if BZ2File is not None:
+            try:
+                return BZ2File(self.getFilename() + ".bz2", "r")
+            except IOError:
+                pass
+        if GzipFile is not None:
+            try:
+                return GzipFile(self.getFilename() + ".gz", "r")
+            except IOError:
+                pass
         return open(self.getFilename(), "r")
 
     def getText(self):
@@ -284,9 +337,13 @@ class LogFile:
         # yield() calls.
 
         f = self.getFile()
-        offset = 0
-        f.seek(0, 2)
-        remaining = f.tell()
+        if not self.finished:
+            offset = 0
+            f.seek(0, 2)
+            remaining = f.tell()
+        else:
+            offset = 0
+            remaining = None
 
         leftover = None
         if self.runEntries and (not channels or
@@ -304,8 +361,12 @@ class LogFile:
         chunks = []
         p = LogFileScanner(chunks.append, channels)
         f.seek(offset)
-        data = f.read(min(remaining, self.BUFFERSIZE))
-        remaining -= len(data)
+        if remaining is not None:
+            data = f.read(min(remaining, self.BUFFERSIZE))
+            remaining -= len(data)
+        else:
+            data = f.read(self.BUFFERSIZE)
+
         offset = f.tell()
         while data:
             p.dataReceived(data)
@@ -316,8 +377,11 @@ class LogFile:
                 else:
                     yield (channel, text)
             f.seek(offset)
-            data = f.read(min(remaining, self.BUFFERSIZE))
-            remaining -= len(data)
+            if remaining is not None:
+                data = f.read(min(remaining, self.BUFFERSIZE))
+                remaining -= len(data)
+            else:
+                data = f.read(self.BUFFERSIZE)
             offset = f.tell()
         del f
 
@@ -379,6 +443,33 @@ class LogFile:
 
     def addEntry(self, channel, text):
         assert not self.finished
+
+        if isinstance(text, unicode):
+            text = text.encode('utf-8')
+        if channel != HEADER:
+            # Truncate the log if it's more than logMaxSize bytes
+            if self.logMaxSize and self.nonHeaderLength > self.logMaxSize:
+                # Add a message about what's going on
+                if not self.maxLengthExceeded:
+                    msg = "\nOutput exceeded %i bytes, remaining output has been truncated\n" % self.logMaxSize
+                    self.addEntry(HEADER, msg)
+                    self.merge()
+                    self.maxLengthExceeded = True
+
+                if self.logMaxTailSize:
+                    # Update the tail buffer
+                    self.tailBuffer.append((channel, text))
+                    self.tailLength += len(text)
+                    while self.tailLength > self.logMaxTailSize:
+                        # Drop some stuff off the beginning of the buffer
+                        c,t = self.tailBuffer.pop(0)
+                        n = len(t)
+                        self.tailLength -= n
+                        assert self.tailLength >= 0
+                return
+
+            self.nonHeaderLength += len(text)
+
         # we only add to .runEntries here. merge() is responsible for adding
         # merged chunks to .entries
         if self.runEntries and channel != self.runEntries[0][0]:
@@ -400,14 +491,24 @@ class LogFile:
         self.addEntry(HEADER, text)
 
     def finish(self):
-        self.merge()
+        if self.tailBuffer:
+            msg = "\nFinal %i bytes follow below:\n" % self.tailLength
+            tmp = self.runEntries
+            self.runEntries = [(HEADER, msg)]
+            self.merge()
+            self.runEntries = self.tailBuffer
+            self.merge()
+            self.runEntries = tmp
+            self.merge()
+            self.tailBuffer = []
+        else:
+            self.merge()
+
         if self.openfile:
             # we don't do an explicit close, because there might be readers
             # shareing the filehandle. As soon as they stop reading, the
-            # filehandle will be released and automatically closed. We will
-            # do a sync, however, to make sure the log gets saved in case of
-            # a crash.
-            os.fsync(self.openfile.fileno())
+            # filehandle will be released and automatically closed.
+            self.openfile.flush()
             del self.openfile
         self.finished = True
         watchers = self.finishedWatchers
@@ -416,6 +517,55 @@ class LogFile:
             w.callback(self)
         self.watchers = []
 
+
+    def compressLog(self):
+        # bail out if there's no compression support
+        if self.compressMethod == "bz2":
+            if BZ2File is None:
+                return
+            compressed = self.getFilename() + ".bz2.tmp"
+        elif self.compressMethod == "gz":
+            if GzipFile is None:
+                return
+            compressed = self.getFilename() + ".gz.tmp"
+        d = threads.deferToThread(self._compressLog, compressed)
+        d.addCallback(self._renameCompressedLog, compressed)
+        d.addErrback(self._cleanupFailedCompress, compressed)
+        return d
+
+    def _compressLog(self, compressed):
+        infile = self.getFile()
+        if self.compressMethod == "bz2":
+            cf = BZ2File(compressed, 'w')
+        elif self.compressMethod == "gz":
+            cf = GzipFile(compressed, 'w')
+        bufsize = 1024*1024
+        while True:
+            buf = infile.read(bufsize)
+            cf.write(buf)
+            if len(buf) < bufsize:
+                break
+        cf.close()
+    def _renameCompressedLog(self, rv, compressed):
+        if self.compressMethod == "bz2":
+            filename = self.getFilename() + '.bz2'
+        else:
+            filename = self.getFilename() + '.gz'
+        if sys.platform == 'win32':
+            # windows cannot rename a file on top of an existing one, so
+            # fall back to delete-first. There are ways this can fail and
+            # lose the builder's history, so we avoid using it in the
+            # general (non-windows) case
+            if os.path.exists(filename):
+                os.unlink(filename)
+        os.rename(compressed, filename)
+        _tryremove(self.getFilename(), 1, 5)
+    def _cleanupFailedCompress(self, failure, compressed):
+        log.msg("failed to compress %s" % self.getFilename())
+        if os.path.exists(compressed):
+            _tryremove(compressed, 1, 5)
+        failure.trap() # reraise the failure
+
     # persistence stuff
     def __getstate__(self):
         d = self.__dict__.copy()
@@ -502,15 +652,12 @@ class Event:
     started = None
     finished = None
     text = []
-    color = None
 
     # IStatusEvent methods
     def getTimes(self):
         return (self.started, self.finished)
     def getText(self):
         return self.text
-    def getColor(self):
-        return self.color
     def getLogs(self):
         return []
 
@@ -614,6 +761,7 @@ class BuildRequestStatus:
         self.builderName = builderName
         self.builds = [] # list of BuildStatus objects
         self.observers = []
+        self.submittedAt = None
 
     def buildStarted(self, build):
         self.builds.append(build)
@@ -635,6 +783,11 @@ class BuildRequestStatus:
     def unsubscribe(self, observer):
         self.observers.remove(observer)
 
+    def getSubmitTime(self):
+        return self.submittedAt
+    def setSubmitTime(self, t):
+        self.submittedAt = t
+
 
 class BuildStepStatus(styles.Versioned):
     """
@@ -646,11 +799,6 @@ class BuildStepStatus(styles.Versioned):
     tests might gather statistics about the number of passed, failed,
     or skipped tests.
 
-    @type color: string
-    @cvar color: color that this step feels best represents its
-                 current mood. yellow,green,red,orange are the
-                 most likely choices, although purple indicates
-                 an exception
     @type progress: L{buildbot.status.progress.StepProgress}
     @cvar progress: tracks ETA for the step
     @type text: list of strings
@@ -671,7 +819,6 @@ class BuildStepStatus(styles.Versioned):
     finished = None
     progress = None
     text = []
-    color = None
     results = (None, [])
     text2 = []
     watchers = []
@@ -718,6 +865,9 @@ class BuildStepStatus(styles.Versioned):
     def getURLs(self):
         return self.urls.copy()
 
+    def isStarted(self):
+        return (self.started is not None)
+
     def isFinished(self):
         return (self.finished is not None)
 
@@ -751,12 +901,6 @@ class BuildStepStatus(styles.Versioned):
         presenting them to the user."""
         return self.text
 
-    def getColor(self):
-        """Returns a single string with the color that should be used to
-        display this step. 'green', 'orange', 'red', 'yellow' and 'purple'
-        are the most likely ones."""
-        return self.color
-
     def getResults(self):
         """Return a tuple describing the results of the step.
         'result' is one of the constants in L{buildbot.status.builder}:
@@ -815,6 +959,9 @@ class BuildStepStatus(styles.Versioned):
     def setName(self, stepname):
         self.name = stepname
 
+    def setColor(self, color):
+        log.msg("BuildStepStatus.setColor is no longer supported -- ignoring color %s" % (color,))
+
     def setProgress(self, stepprogress):
         self.progress = stepprogress
 
@@ -827,6 +974,9 @@ class BuildStepStatus(styles.Versioned):
         assert self.started # addLog before stepStarted won't notify watchers
         logfilename = self.build.generateLogfileName(self.name, name)
         log = LogFile(self, name, logfilename)
+        log.logMaxSize = self.build.builder.logMaxSize
+        log.logMaxTailSize = self.build.builder.logMaxTailSize
+        log.compressMethod = self.build.builder.logCompressionMethod
         self.logs.append(log)
         for w in self.watchers:
             receiver = w.logStarted(self.build, self, log)
@@ -858,12 +1008,14 @@ class BuildStepStatus(styles.Versioned):
     def addURL(self, name, url):
         self.urls[name] = url
 
-    def setColor(self, color):
-        self.color = color
     def setText(self, text):
         self.text = text
+        for w in self.watchers:
+            w.stepTextChanged(self.build, self, text)
     def setText2(self, text):
         self.text2 = text
+        for w in self.watchers:
+            w.stepText2Changed(self.build, self, text)
 
     def setStatistic(self, name, value):
         """Set the given statistic.  Usually called by subclasses.
@@ -873,9 +1025,19 @@ class BuildStepStatus(styles.Versioned):
     def stepFinished(self, results):
         self.finished = util.now()
         self.results = results
+        cld = [] # deferreds for log compression
+        logCompressionLimit = self.build.builder.logCompressionLimit
         for loog in self.logs:
             if not loog.isFinished():
                 loog.finish()
+            # if log compression is on, and it's a real LogFile,
+            # HTMLLogFiles aren't files
+            if logCompressionLimit is not False and \
+                    isinstance(loog, LogFile):
+                if os.path.getsize(loog.getFilename()) > logCompressionLimit:
+                    loog_deferred = loog.compressLog()
+                    if loog_deferred:
+                        cld.append(loog_deferred)
 
         for r in self.updates.keys():
             if self.updates[r] is not None:
@@ -886,6 +1048,12 @@ class BuildStepStatus(styles.Versioned):
         self.finishedWatchers = []
         for w in watchers:
             w.callback(self)
+        if cld:
+            return defer.DeferredList(cld)
+
+    def checkLogfiles(self):
+        # filter out logs that have been deleted
+        self.logs = [ l for l in self.logs if l.hasContents() ]
 
     # persistence
 
@@ -902,6 +1070,8 @@ class BuildStepStatus(styles.Versioned):
     def __setstate__(self, d):
         styles.Versioned.__setstate__(self, d)
         # self.build must be filled in by our parent
+
+        # point the logs to this object
         for loog in self.logs:
             loog.step = self
 
@@ -922,12 +1092,12 @@ class BuildStatus(styles.Versioned):
     reason = None
     changes = []
     blamelist = []
+    requests = []
     progress = None
     started = None
     finished = None
     currentStep = None
     text = []
-    color = None
     results = None
     slavename = "???"
 
@@ -953,6 +1123,10 @@ class BuildStatus(styles.Versioned):
         self.steps = []
         self.testResults = {}
         self.properties = Properties()
+        self.requests = []
+
+    def __repr__(self):
+        return "<%s #%s>" % (self.__class__.__name__, self.number)
 
     # IBuildStatus
 
@@ -987,12 +1161,15 @@ class BuildStatus(styles.Versioned):
     def getChanges(self):
         return self.changes
 
+    def getRequests(self):
+        return self.requests
+
     def getResponsibleUsers(self):
         return self.blamelist
 
     def getInterestedUsers(self):
         # TODO: the Builder should add others: sheriffs, domain-owners
-        return self.blamelist
+        return self.blamelist + self.properties.getProperty('owners', [])
 
     def getSteps(self):
         """Return a list of IBuildStepStatus objects. For invariant builds
@@ -1059,9 +1236,6 @@ class BuildStatus(styles.Versioned):
             text.extend(s.text2)
         return text
 
-    def getColor(self):
-        return self.color
-
     def getResults(self):
         return self.results
 
@@ -1132,6 +1306,9 @@ class BuildStatus(styles.Versioned):
         self.source = sourceStamp
         self.changes = self.source.changes
 
+    def setRequests(self, requests):
+        self.requests = requests
+
     def setReason(self, reason):
         self.reason = reason
     def setBlamelist(self, blamelist):
@@ -1154,8 +1331,6 @@ class BuildStatus(styles.Versioned):
     def setText(self, text):
         assert isinstance(text, (list, tuple))
         self.text = text
-    def setColor(self, color):
-        self.color = color
     def setResults(self, results):
         self.results = results
 
@@ -1197,12 +1372,6 @@ class BuildStatus(styles.Versioned):
 
     # methods called by our BuilderStatus parent
 
-    def pruneLogs(self):
-        # this build is somewhat old: remove the build logs to save space
-        # TODO: delete logs visible through IBuildStatus.getLogs
-        for s in self.steps:
-            s.pruneLogs()
-
     def pruneSteps(self):
         # this build is very old: remove the build steps too
         self.steps = []
@@ -1246,10 +1415,8 @@ class BuildStatus(styles.Versioned):
             # was interrupted. The builder will have a 'shutdown' event, but
             # someone looking at just this build will be confused as to why
             # the last log is truncated.
-        del d['builder'] # filled in by our parent when loading
-        del d['watchers']
-        del d['updates']
-        del d['finishedWatchers']
+        for k in 'builder', 'watchers', 'updates', 'requests', 'finishedWatchers':
+            if k in d: del d[k]
         return d
 
     def __setstate__(self, d):
@@ -1298,6 +1465,12 @@ class BuildStatus(styles.Versioned):
                     # transferring its contents onto disk if necessary
                     l.upgrade(logfilename)
 
+    def checkLogfiles(self):
+        # check that all logfiles exist, and remove references to any that
+        # have been deleted (e.g., by purge())
+        for s in self.steps:
+            s.checkLogfiles()
+
     def saveYourself(self):
         filename = os.path.join(self.builder.basedir, "%d" % self.number)
         if os.path.isdir(filename):
@@ -1346,9 +1519,12 @@ class BuilderStatus(styles.Versioned):
     # these limit the amount of memory we consume, as well as the size of the
     # main Builder pickle. The Build and LogFile pickles on disk must be
     # handled separately.
-    buildCacheSize = 30
+    buildCacheSize = 15
+    eventHorizon = 50 # forget events beyond this
+
+    # these limit on-disk storage
+    logHorizon = 40 # forget logs in steps in builds beyond this
     buildHorizon = 100 # forget builds beyond this
-    stepHorizon = 50 # forget steps in builds beyond this
 
     category = None
     currentBigState = "offline" # or idle/waiting/interlocked/building
@@ -1369,7 +1545,12 @@ class BuilderStatus(styles.Versioned):
         self.pendingBuilds = []
         self.nextBuild = None
         self.watchers = []
-        self.buildCache = [] # TODO: age builds out of the cache
+        self.buildCache = weakref.WeakValueDictionary()
+        self.buildCache_LRU = []
+        self.logCompressionLimit = False # default to no compression for tests
+        self.logCompressionMethod = "bz2"
+        self.logMaxSize = None # No default limit
+        self.logMaxTailSize = None # No tail buffering
 
     # persistence
 
@@ -1381,6 +1562,7 @@ class BuilderStatus(styles.Versioned):
         d = styles.Versioned.__getstate__(self)
         d['watchers'] = []
         del d['buildCache']
+        del d['buildCache_LRU']
         for b in self.currentBuilds:
             b.saveYourself()
             # TODO: push a 'hey, build was interrupted' event
@@ -1396,7 +1578,8 @@ class BuilderStatus(styles.Versioned):
         # when loading, re-initialize the transient stuff. Remember that
         # upgradeToVersion1 and such will be called after this finishes.
         styles.Versioned.__setstate__(self, d)
-        self.buildCache = []
+        self.buildCache = weakref.WeakValueDictionary()
+        self.buildCache_LRU = []
         self.currentBuilds = []
         self.pendingBuilds = []
         self.watchers = []
@@ -1404,6 +1587,18 @@ class BuilderStatus(styles.Versioned):
         # self.basedir must be filled in by our parent
         # self.status must be filled in by our parent
 
+    def reconfigFromBuildmaster(self, buildmaster):
+        # Note that we do not hang onto the buildmaster, since this object
+        # gets pickled and unpickled.
+        if buildmaster.buildCacheSize:
+            self.buildCacheSize = buildmaster.buildCacheSize
+        if buildmaster.eventHorizon:
+            self.eventHorizon = buildmaster.eventHorizon
+        if buildmaster.logHorizon:
+            self.logHorizon = buildmaster.logHorizon
+        if buildmaster.buildHorizon:
+            self.buildHorizon = buildmaster.buildHorizon
+
     def upgradeToVersion1(self):
         if hasattr(self, 'slavename'):
             self.slavenames = [self.slavename]
@@ -1425,8 +1620,21 @@ class BuilderStatus(styles.Versioned):
         else:
             self.nextBuildNumber = 0
 
+    def setLogCompressionLimit(self, lowerLimit):
+        self.logCompressionLimit = lowerLimit
+
+    def setLogCompressionMethod(self, method):
+        assert method in ("bz2", "gz")
+        self.logCompressionMethod = method
+
+    def setLogMaxSize(self, upperLimit):
+        self.logMaxSize = upperLimit
+
+    def setLogMaxTailSize(self, tailSize):
+        self.logMaxTailSize = tailSize
+
     def saveYourself(self):
-        for b in self.buildCache:
+        for b in self.currentBuilds:
             if not b.isFinished:
                 # interrupted build, need to save it anyway.
                 # BuildStatus.saveYourself will mark it as interrupted.
@@ -1447,41 +1655,94 @@ class BuilderStatus(styles.Versioned):
 
     # build cache management
 
-    def addBuildToCache(self, build):
-        if build in self.buildCache:
-            return
-        self.buildCache.append(build)
-        while len(self.buildCache) > self.buildCacheSize:
-            self.buildCache.pop(0)
+    def makeBuildFilename(self, number):
+        return os.path.join(self.basedir, "%d" % number)
+
+    def touchBuildCache(self, build):
+        self.buildCache[build.number] = build
+        if build in self.buildCache_LRU:
+            self.buildCache_LRU.remove(build)
+        self.buildCache_LRU = self.buildCache_LRU[-(self.buildCacheSize-1):] + [ build ]
+        return build
 
     def getBuildByNumber(self, number):
+        # first look in currentBuilds
         for b in self.currentBuilds:
             if b.number == number:
-                return b
-        for build in self.buildCache:
-            if build.number == number:
-                return build
-        filename = os.path.join(self.basedir, "%d" % number)
+                return self.touchBuildCache(b)
+
+        # then in the buildCache
+        if number in self.buildCache:
+            return self.touchBuildCache(self.buildCache[number])
+
+        # then fall back to loading it from disk
+        filename = self.makeBuildFilename(number)
         try:
+            log.msg("Loading builder %s's build %d from on-disk pickle"
+                % (self.name, number))
             build = load(open(filename, "rb"))
             styles.doUpgrade()
             build.builder = self
             # handle LogFiles from after 0.5.0 and before 0.6.5
             build.upgradeLogfiles()
-            self.addBuildToCache(build)
-            return build
+            # check that logfiles exist
+            build.checkLogfiles()
+            return self.touchBuildCache(build)
         except IOError:
             raise IndexError("no such build %d" % number)
         except EOFError:
             raise IndexError("corrupted build pickle %d" % number)
 
     def prune(self):
-        return # TODO: change this to walk through the filesystem
-        # first, blow away all builds beyond our build horizon
-        self.builds = self.builds[-self.buildHorizon:]
-        # then prune steps in builds past the step horizon
-        for b in self.builds[0:-self.stepHorizon]:
-            b.pruneSteps()
+        gc.collect()
+
+        # begin by pruning our own events
+        self.events = self.events[-self.eventHorizon:]
+
+        # get the horizons straight
+        if self.buildHorizon:
+            earliest_build = self.nextBuildNumber - self.buildHorizon
+        else:
+            earliest_build = 0
+
+        if self.logHorizon:
+            earliest_log = self.nextBuildNumber - self.logHorizon
+        else:
+            earliest_log = 0
+
+        if earliest_log < earliest_build:
+            earliest_log = earliest_build
+
+        if earliest_build == 0:
+            return
+
+        # skim the directory and delete anything that shouldn't be there anymore
+        build_re = re.compile(r"^([0-9]+)$")
+        build_log_re = re.compile(r"^([0-9]+)-.*$")
+        # if the directory doesn't exist, bail out here
+        if not os.path.exists(self.basedir):
+            return
+
+        for filename in os.listdir(self.basedir):
+            num = None
+            mo = build_re.match(filename)
+            is_logfile = False
+            if mo:
+                num = int(mo.group(1))
+            else:
+                mo = build_log_re.match(filename)
+                if mo:
+                    num = int(mo.group(1))
+                    is_logfile = True
+
+            if num is None: continue
+            if num in self.buildCache: continue
+
+            if (is_logfile and num < earliest_log) or num < earliest_build:
+                pathname = os.path.join(self.basedir, filename)
+                log.msg("pruning '%s'" % pathname)
+                try: os.unlink(pathname)
+                except OSError: pass
 
     # IBuilderStatus methods
     def getName(self):
@@ -1505,6 +1766,9 @@ class BuilderStatus(styles.Versioned):
             b = self.getBuild(-2)
         return b
 
+    def getCategory(self):
+        return self.category
+
     def getBuild(self, number):
         if number < 0:
             number = self.nextBuildNumber + number
@@ -1554,7 +1818,7 @@ class BuilderStatus(styles.Versioned):
                 if got >= num_builds:
                     return
 
-    def eventGenerator(self, branches=[]):
+    def eventGenerator(self, branches=[], categories=[]):
         """This function creates a generator which will provide all of this
         Builder's status events, starting with the most recent and
         progressing backwards in time. """
@@ -1573,9 +1837,16 @@ class BuilderStatus(styles.Versioned):
         for Nb in range(1, self.nextBuildNumber+1):
             b = self.getBuild(-Nb)
             if not b:
+                # HACK: If this is the first build we are looking at, it is
+                # possible it's in progress but locked before it has written a
+                # pickle; in this case keep looking.
+                if Nb == 1:
+                    continue
                 break
             if branches and not b.getSourceStamp().branch in branches:
                 continue
+            if categories and not b.getBuilder().getCategory() in categories:
+                continue
             steps = b.getSteps()
             for Ns in range(1, len(steps)+1):
                 if steps[-Ns].started:
@@ -1604,24 +1875,22 @@ class BuilderStatus(styles.Versioned):
     def setSlavenames(self, names):
         self.slavenames = names
 
-    def addEvent(self, text=[], color=None):
+    def addEvent(self, text=[]):
         # this adds a duration event. When it is done, the user should call
-        # e.finish(). They can also mangle it by modifying .text and .color
+        # e.finish(). They can also mangle it by modifying .text
         e = Event()
         e.started = util.now()
         e.text = text
-        e.color = color
         self.events.append(e)
         return e # they are free to mangle it further
 
-    def addPointEvent(self, text=[], color=None):
+    def addPointEvent(self, text=[]):
         # this adds a point event, one which occurs as a single atomic
         # instant of time.
         e = Event()
         e.started = util.now()
         e.finished = 0
         e.text = text
-        e.color = color
         self.events.append(e)
         return e # for consistency, but they really shouldn't touch it
 
@@ -1639,7 +1908,11 @@ class BuilderStatus(styles.Versioned):
             target.builderChangedState(self.name, state)
             return
         for w in self.watchers:
-            w.builderChangedState(self.name, state)
+            try:
+                w.builderChangedState(self.name, state)
+            except:
+                log.msg("Exception caught publishing state to %r" % w)
+                log.err()
 
     def newBuild(self):
         """The Builder has decided to start a build, but the Build object is
@@ -1657,8 +1930,14 @@ class BuilderStatus(styles.Versioned):
 
     def addBuildRequest(self, brstatus):
         self.pendingBuilds.append(brstatus)
-    def removeBuildRequest(self, brstatus):
+        for w in self.watchers:
+            w.requestSubmitted(brstatus)
+
+    def removeBuildRequest(self, brstatus, cancelled=False):
         self.pendingBuilds.remove(brstatus)
+        if cancelled:
+            for w in self.watchers:
+                w.requestCancelled(self, brstatus)
 
     # buildStarted is called by our child BuildStatus instances
     def buildStarted(self, s):
@@ -1669,21 +1948,24 @@ class BuilderStatus(styles.Versioned):
         assert s.number == self.nextBuildNumber - 1
         assert s not in self.currentBuilds
         self.currentBuilds.append(s)
-        self.addBuildToCache(s)
+        self.touchBuildCache(s)
 
         # now that the BuildStatus is prepared to answer queries, we can
         # announce the new build to all our watchers
 
         for w in self.watchers: # TODO: maybe do this later? callLater(0)?
-            receiver = w.buildStarted(self.getName(), s)
-            if receiver:
-                if type(receiver) == type(()):
-                    s.subscribe(receiver[0], receiver[1])
-                else:
-                    s.subscribe(receiver)
-                d = s.waitUntilFinished()
-                d.addCallback(lambda s: s.unsubscribe(receiver))
-
+            try:
+                receiver = w.buildStarted(self.getName(), s)
+                if receiver:
+                    if type(receiver) == type(()):
+                        s.subscribe(receiver[0], receiver[1])
+                    else:
+                        s.subscribe(receiver)
+                    d = s.waitUntilFinished()
+                    d.addCallback(lambda s: s.unsubscribe(receiver))
+            except:
+                log.msg("Exception caught notifying %r of buildStarted event" % w)
+                log.err()
 
     def _buildFinished(self, s):
         assert s in self.currentBuilds
@@ -1693,7 +1975,11 @@ class BuilderStatus(styles.Versioned):
         name = self.getName()
         results = s.getResults()
         for w in self.watchers:
-            w.buildFinished(name, s, results)
+            try:
+                w.buildFinished(name, s, results)
+            except:
+                log.msg("Exception caught notifying %r of buildFinished event" % w)
+                log.err()
 
         self.prune() # conserve disk
 
@@ -1782,12 +2068,16 @@ class SlaveStatus:
 
     admin = None
     host = None
+    access_uri = None
+    version = None
     connected = False
+    graceful_shutdown = False
 
     def __init__(self, name):
         self.name = name
         self._lastMessageReceived = 0
         self.runningBuilds = []
+        self.graceful_callbacks = []
 
     def getName(self):
         return self.name
@@ -1795,6 +2085,10 @@ class SlaveStatus:
         return self.admin
     def getHost(self):
         return self.host
+    def getAccessURI(self):
+        return self.access_uri
+    def getVersion(self):
+        return self.version
     def isConnected(self):
         return self.connected
     def lastMessageReceived(self):
@@ -1806,6 +2100,10 @@ class SlaveStatus:
         self.admin = admin
     def setHost(self, host):
         self.host = host
+    def setAccessURI(self, access_uri):
+        self.access_uri = access_uri
+    def setVersion(self, version):
+        self.version = version
     def setConnected(self, isConnected):
         self.connected = isConnected
     def setLastMessageReceived(self, when):
@@ -1816,6 +2114,25 @@ class SlaveStatus:
     def buildFinished(self, build):
         self.runningBuilds.remove(build)
 
+    def getGraceful(self):
+        """Return the graceful shutdown flag"""
+        return self.graceful_shutdown
+    def setGraceful(self, graceful):
+        """Set the graceful shutdown flag, and notify all the watchers"""
+        self.graceful_shutdown = graceful
+        for cb in self.graceful_callbacks:
+            reactor.callLater(0, cb, graceful)
+    def addGracefulWatcher(self, watcher):
+        """Add watcher to the list of watchers to be notified when the
+        graceful shutdown flag is changed."""
+        if not watcher in self.graceful_callbacks:
+            self.graceful_callbacks.append(watcher)
+    def removeGracefulWatcher(self, watcher):
+        """Remove watcher from the list of watchers to be notified when the
+        graceful shutdown flag is changed."""
+        if watcher in self.graceful_callbacks:
+            self.graceful_callbacks.remove(watcher)
+
 class Status:
     """
     I represent the status of the buildmaster.
@@ -1841,6 +2158,12 @@ class Status:
         self.watchers = []
         self.activeBuildSets = []
         assert os.path.isdir(basedir)
+        # compress logs bigger than 4k, a good default on linux
+        self.logCompressionLimit = 4*1024
+        self.logCompressionMethod = "bz2"
+        # No default limit to the log size
+        self.logMaxSize = None
+        self.logMaxTailSize = None
 
 
     # methods called by our clients
@@ -1904,11 +2227,11 @@ class Status:
                     break
             else:
                 return None
-            return prefix + "builders/%s/builds/%d/steps/%s/logs/%d" % (
+            return prefix + "builders/%s/builds/%d/steps/%s/logs/%s" % (
                 urllib.quote(builder.getName(), safe=''),
                 build.getNumber(),
                 urllib.quote(step.getName(), safe=''),
-                lognum)
+                urllib.quote(log.getName()))
 
     def getChangeSources(self):
         return list(self.botmaster.parent.change_svc)
@@ -2049,10 +2372,14 @@ class Status:
         builder_status.status = self
 
         if not os.path.isdir(builder_status.basedir):
-            os.mkdir(builder_status.basedir)
+            os.makedirs(builder_status.basedir)
         builder_status.determineNextBuildNumber()
 
         builder_status.setBigState("offline")
+        builder_status.setLogCompressionLimit(self.logCompressionLimit)
+        builder_status.setLogCompressionMethod(self.logCompressionMethod)
+        builder_status.setLogMaxSize(self.logMaxSize)
+        builder_status.setLogMaxTailSize(self.logMaxTailSize)
 
         for t in self.watchers:
             self.announceNewBuilder(t, name, builder_status)
@@ -2063,10 +2390,6 @@ class Status:
         for t in self.watchers:
             t.builderRemoved(name)
 
-    def prune(self):
-        for b in self.botmaster.builders.values():
-            b.builder_status.prune()
-
     def buildsetSubmitted(self, bss):
         self.activeBuildSets.append(bss)
         bss.waitUntilFinished().addCallback(self.activeBuildSets.remove)
