@@ -3,6 +3,8 @@ from __future__ import generators
 import sys, time, os.path
 import urllib
 
+from twisted.web import html, resource
+
 from buildbot import util
 from buildbot import version
 from buildbot.status.web.base import HtmlResource
@@ -22,18 +24,7 @@ else:
 
 class ANYBRANCH: pass # a flag value, used below
 
-class GridStatusResource(HtmlResource):
-    # TODO: docs
-    status = None
-    control = None
-    changemaster = None
-
-    def __init__(self, allowForce=True, css=None):
-        HtmlResource.__init__(self)
-
-        self.allowForce = allowForce
-        self.css = css or grid_css
-
+class GridStatusMixin(object):
     def getTitle(self, request):
         status = self.getStatus(request)
         p = status.getProjectName()
@@ -137,85 +128,13 @@ class GridStatusResource(HtmlResource):
         return '<td valign="bottom" class="sourcestamp">%s</td>\n' % \
             "<br />".join(text)
 
-    def body(self, request):
-        "This method builds the main waterfall display."
+    def getSourceStampKey(self, ss):
+        """Given two source stamps, we want to assign them to the same row if
+        they are the same version of code, even if they differ in minor detail.
 
-        # get url parameters
-        numBuilds = int(request.args.get("width", [5])[0])
-        categories = request.args.get("category", [])
-        branch = request.args.get("branch", [ANYBRANCH])[0]
-        if branch == 'trunk': branch = None
-
-        # and the data we want to render
-        status = self.getStatus(request)
-        stamps = self.getRecentSourcestamps(status, numBuilds, categories, branch)
-
-        projectURL = status.getProjectURL()
-        projectName = status.getProjectName()
-
-        data = '<table class="Grid" border="0" cellspacing="0">\n'
-        data += '<tr>\n'
-        data += '<td class="title"><a href="%s">%s</a>' % (projectURL, projectName)
-        if categories:
-            if len(categories) > 1:
-                data += '\n<br /><b>Categories:</b><br/>%s' % ('<br/>'.join(categories))
-            else:
-                data += '\n<br /><b>Category:</b> %s' % categories[0]
-        if branch != ANYBRANCH:
-            data += '\n<br /><b>Branch:</b> %s' % (branch or 'trunk')
-        data += '</td>\n'
-        for stamp in stamps:
-            data += self.stamp_td(stamp)
-        data += '</tr>\n'
-
-        sortedBuilderNames = status.getBuilderNames()[:]
-        sortedBuilderNames.sort()
-        for bn in sortedBuilderNames:
-            builds = [None] * len(stamps)
-
-            builder = status.getBuilder(bn)
-            if categories and builder.category not in categories:
-                continue
-
-            build = builder.getBuild(-1)
-            while build and None in builds:
-                ss = build.getSourceStamp(absolute=True)
-                for i in range(len(stamps)):
-                    if ss == stamps[i] and builds[i] is None:
-                        builds[i] = build
-                build = build.getPreviousBuild()
-
-            data += '<tr>\n'
-            data += self.builder_td(request, builder)
-            for build in builds:
-                data += self.build_td(request, build)
-            data += '</tr>\n'
-
-        data += '</table>\n'
-
-        # TODO: this stuff should be generated by a template of some sort
-        data += '<hr /><div class="footer">\n'
-
-        welcomeurl = self.path_to_root(request) + "index.html"
-        data += '[<a href="%s">welcome</a>]\n' % welcomeurl
-        data += "<br />\n"
-
-        data += '<a href="http://buildbot.sourceforge.net/">Buildbot</a>'
-        data += "-%s " % version
-        if projectName:
-            data += "working for the "
-            if projectURL:
-                data += "<a href=\"%s\">%s</a> project." % (projectURL,
-                                                            projectName)
-            else:
-                data += "%s project." % projectName
-        data += "<br />\n"
-        data += ("Page built: " +
-                 time.strftime("%a %d %b %Y %H:%M:%S",
-                               time.localtime(util.now()))
-                 + "\n")
-        data += '</div>\n'
-        return data
+        This function returns an appropriate comparison key for that.
+        """
+        return (ss.branch, ss.revision, ss.patch)
 
     def getRecentSourcestamps(self, status, numBuilds, categories, branch):
         """
@@ -240,13 +159,173 @@ class GridStatusResource(HtmlResource):
                 # skip non-matching branches
                 if branch != ANYBRANCH and ss.branch != branch: continue
 
-                sourcestamps[ss] = min(sourcestamps.get(ss, sys.maxint), start)
+                key= self.getSourceStampKey(ss)
+                if key not in sourcestamps or sourcestamps[key][1] > start:
+                    sourcestamps[key] = (ss, start)
 
         # now sort those and take the NUMBUILDS most recent
-        sourcestamps = sourcestamps.items()
+        sourcestamps = sourcestamps.values()
         sourcestamps.sort(lambda x, y: cmp(x[1], y[1]))
         sourcestamps = map(lambda tup : tup[0], sourcestamps)
         sourcestamps = sourcestamps[-numBuilds:]
 
         return sourcestamps
 
+class GridStatusResource(HtmlResource, GridStatusMixin):
+    # TODO: docs
+    status = None
+    control = None
+    changemaster = None
+
+    def __init__(self, allowForce=True, css=None):
+        HtmlResource.__init__(self)
+
+        self.allowForce = allowForce
+        self.css = css or grid_css
+
+
+    def body(self, request):
+        """This method builds the regular grid display.
+        That is, build stamps across the top, build hosts down the left side
+        """
+
+        # get url parameters
+        numBuilds = int(request.args.get("width", [5])[0])
+        categories = request.args.get("category", [])
+        branch = request.args.get("branch", [ANYBRANCH])[0]
+        if branch == 'trunk': branch = None
+
+        # and the data we want to render
+        status = self.getStatus(request)
+        stamps = self.getRecentSourcestamps(status, numBuilds, categories, branch)
+
+        projectURL = status.getProjectURL()
+        projectName = status.getProjectName()
+
+        data = '<table class="Grid" border="0" cellspacing="0">\n'
+        data += '<tr>\n'
+        data += '<td class="title"><a href="%s">%s</a>' % (projectURL, projectName)
+        if categories:
+            html_categories = map(html.escape, categories)
+            if len(categories) > 1:
+                data += '\n<br /><b>Categories:</b><br/>%s' % ('<br/>'.join(html_categories))
+            else:
+                data += '\n<br /><b>Category:</b> %s' % html_categories[0]
+        if branch != ANYBRANCH:
+            data += '\n<br /><b>Branch:</b> %s' % (html.escape(branch or 'trunk'))
+        data += '</td>\n'
+        for stamp in stamps:
+            data += self.stamp_td(stamp)
+        data += '</tr>\n'
+
+        sortedBuilderNames = status.getBuilderNames()[:]
+        sortedBuilderNames.sort()
+        for bn in sortedBuilderNames:
+            builds = [None] * len(stamps)
+
+            builder = status.getBuilder(bn)
+            if categories and builder.category not in categories:
+                continue
+
+            build = builder.getBuild(-1)
+            while build and None in builds:
+                ss = build.getSourceStamp(absolute=True)
+                key= self.getSourceStampKey(ss)
+                for i in range(len(stamps)):
+                    if key == self.getSourceStampKey(stamps[i]) and builds[i] is None:
+                        builds[i] = build
+                build = build.getPreviousBuild()
+
+            data += '<tr>\n'
+            data += self.builder_td(request, builder)
+            for build in builds:
+                data += self.build_td(request, build)
+            data += '</tr>\n'
+
+        data += '</table>\n'
+
+        data += self.footer(status, request)
+        return data
+
+class TransposedGridStatusResource(HtmlResource, GridStatusMixin):
+    # TODO: docs
+    status = None
+    control = None
+    changemaster = None
+
+    def __init__(self, allowForce=True, css=None):
+        HtmlResource.__init__(self)
+
+        self.allowForce = allowForce
+        self.css = css or grid_css
+
+
+    def body(self, request):
+        """This method builds the transposed grid display.
+        That is, build hosts across the top, ebuild stamps down the left side
+        """
+
+        # get url parameters
+        numBuilds = int(request.args.get("length", [5])[0])
+        categories = request.args.get("category", [])
+        branch = request.args.get("branch", [ANYBRANCH])[0]
+        if branch == 'trunk': branch = None
+
+        # and the data we want to render
+        status = self.getStatus(request)
+        stamps = self.getRecentSourcestamps(status, numBuilds, categories, branch)
+
+        projectURL = status.getProjectURL()
+        projectName = status.getProjectName()
+
+        data = '<table class="Grid" border="0" cellspacing="0">\n'
+        data += '<tr>\n'
+        data += '<td class="title"><a href="%s">%s</a>' % (projectURL, projectName)
+        if categories:
+            html_categories = map(html.escape, categories)
+            if len(categories) > 1:
+                data += '\n<br /><b>Categories:</b><br/>%s' % ('<br/>'.join(html_categories))
+            else:
+                data += '\n<br /><b>Category:</b> %s' % html_categories[0]
+        if branch != ANYBRANCH:
+            data += '\n<br /><b>Branch:</b> %s' % (html.escape(branch or 'trunk'))
+        data += '</td>\n'
+
+        sortedBuilderNames = status.getBuilderNames()[:]
+        sortedBuilderNames.sort()
+
+        builder_builds = []
+
+        for bn in sortedBuilderNames:
+            builds = [None] * len(stamps)
+
+            builder = status.getBuilder(bn)
+            if categories and builder.category not in categories:
+                continue
+
+            build = builder.getBuild(-1)
+            while build and None in builds:
+                ss = build.getSourceStamp(absolute=True)
+                key = self.getSourceStampKey(ss)
+                for i in range(len(stamps)):
+                    if key == self.getSourceStampKey(stamps[i]) and builds[i] is None:
+                        builds[i] = build
+                build = build.getPreviousBuild()
+
+            data += self.builder_td(request, builder)
+            builder_builds.append(builds)
+
+        data += '</tr>\n'
+
+        for i in range(len(stamps)):
+            data += '<tr>\n'
+            data += self.stamp_td(stamps[i])
+            for builds in builder_builds:
+                data += self.build_td(request, builds[i])
+            data += '</tr>\n'
+
+        data += '</table>\n'
+
+        data += self.footer(status, request)
+        return data
+
