@@ -191,8 +191,8 @@ class SpacerBox(components.Adapter):
         b.spacer = True
         return b
 components.registerAdapter(SpacerBox, Spacer, IBox)
-    
-def insertGaps(g, lastEventTime, idleGap=2):
+
+def insertGaps(g, showEvents, lastEventTime, idleGap=2):
     debug = False
 
     e = g.next()
@@ -212,6 +212,8 @@ def insertGaps(g, lastEventTime, idleGap=2):
 
     while 1:
         e = g.next()
+        if not showEvents and isinstance(e, builder.Event):
+            continue
         starts, finishes = e.getTimes()
         if debug: log.msg("E2", starts, finishes)
         if finishes == 0:
@@ -261,9 +263,9 @@ be displayed that occurred <b>before</b> this timestamp. Instead of providing
 given interval, where each timestamp on the left hand edge counts as a single
 event. You can add a <tt>num_events=</tt> argument to override this this.</p>
 
-<h2>Hiding non-Build events</h2>
+<h2>Showing non-Build events</h2>
 
-<p>By passing <tt>show_events=false</tt>, you can remove the "buildslave
+<p>By passing <tt>show_events=true</tt>, you can add the "buildslave
 attached", "buildslave detached", and "builder reconfigured" events that
 appear in-between the actual builds.</p>
 
@@ -325,13 +327,13 @@ class WaterfallHelp(HtmlResource):
         data = ''
         status = self.getStatus(request)
 
-        showEvents_checked = 'checked="checked"'
-        if request.args.get("show_events", ["true"])[0].lower() == "true":
-            showEvents_checked = ''
+        showEvents_checked = ''
+        if request.args.get("show_events", ["false"])[0].lower() == "true":
+            showEvents_checked = 'checked="checked"'
         show_events_input = ('<p>'
                              '<input type="checkbox" name="show_events" '
-                             'value="false" %s>'
-                             'Hide non-Build events'
+                             'value="true" %s>'
+                             'Show non-Build events'
                              '</p>\n'
                              ) % showEvents_checked
 
@@ -415,9 +417,11 @@ class WaterfallStatusResource(HtmlResource):
     """This builds the main status page, with the waterfall display, and
     all child pages."""
 
-    def __init__(self, categories=None):
+    def __init__(self, categories=None, num_events=200, num_events_max=None):
         HtmlResource.__init__(self)
         self.categories = categories
+        self.num_events=num_events
+        self.num_events_max=num_events_max
         self.putChild("help", WaterfallHelp(categories))
 
     def getTitle(self, request):
@@ -570,7 +574,6 @@ class WaterfallStatusResource(HtmlResource):
 
         data += "</table>\n"
 
-        data += '<hr /><div class="footer">\n'
 
         def with_args(req, remove_args=[], new_args=[], new_path=None):
             # sigh, nevow makes this sort of manipulation easier
@@ -608,32 +611,13 @@ class WaterfallStatusResource(HtmlResource):
         helppage = with_args(request, new_path=helpurl)
         data += '[<a href="%s">help</a>]\n' % helppage
 
-        welcomeurl = self.path_to_root(request) + "index.html"
-        data += '[<a href="%s">welcome</a>]\n' % welcomeurl
-
         if self.get_reload_time(request) is not None:
             no_reload_page = with_args(request, remove_args=["reload"])
             data += '[<a href="%s">Stop Reloading</a>]\n' % no_reload_page
 
         data += "<br />\n"
+	data += self.footer(status, request)
 
-
-        bburl = "http://buildbot.net/?bb-ver=%s" % urllib.quote(version)
-        data += '<a href="%s">Buildbot-%s</a> ' % (bburl, version)
-        if projectName:
-            data += "working for the "
-            if projectURL:
-                data += '<a href="%s">%s</a> project.' % (projectURL,
-                                                            projectName)
-            else:
-                data += "%s project." % projectName
-        data += "<br />\n"
-        # TODO: push this to the right edge, if possible
-        data += ("Page built: " +
-                 time.strftime("%a %d %b %Y %H:%M:%S",
-                               time.localtime(util.now()))
-                 + "\n")
-        data += '</div>\n'
         return data
 
     def body0(self, request, builders):
@@ -684,7 +668,7 @@ class WaterfallStatusResource(HtmlResource):
         # TODO: see if we can use a cached copy
 
         showEvents = False
-        if request.args.get("show_events", ["true"])[0].lower() == "true":
+        if request.args.get("show_events", ["false"])[0].lower() == "true":
             showEvents = True
         filterCategories = request.args.get('category', [])
         filterBranches = [b for b in request.args.get("branch", []) if b]
@@ -697,7 +681,11 @@ class WaterfallStatusResource(HtmlResource):
         else:
             minTime = None
         spanLength = 10  # ten-second chunks
-        maxPageLen = int(request.args.get("num_events", [200])[0])
+        req_events=int(request.args.get("num_events", [self.num_events])[0])
+        if self.num_events_max and req_events > self.num_events_max:
+            maxPageLen = self.num_events_max
+        else:
+            maxPageLen = req_events
 
         # first step is to walk backwards in time, asking each column
         # (commit, all builders) if they have any events there. Build up the
@@ -733,7 +721,10 @@ class WaterfallStatusResource(HtmlResource):
             return event
 
         for s in sources:
-            gen = insertGaps(s.eventGenerator(filterBranches, filterCategories), lastEventTime)
+            gen = insertGaps(s.eventGenerator(filterBranches,
+                                              filterCategories),
+                             showEvents,
+                             lastEventTime)
             sourceGenerators.append(gen)
             # get the first event
             sourceEvents.append(get_event_from(gen))
