@@ -42,7 +42,10 @@ class Source(LoggingBuildStep):
              files are deleted but generated files can influence test
              behavior (e.g. python's .pyc files), or when source
              directories are deleted but generated files prevent CVS from
-             removing them.
+             removing them. When used with a patched checkout, from a
+             previous buildbot try for instance, it will try to "revert"
+             the changes first and will do a clobber if it is unable to
+             get a clean checkout. The behavior is SCM-dependent.
 
            - 'copy': specifies that the source-controlled workspace
              should be maintained in a separate directory (called the
@@ -176,11 +179,14 @@ class Source(LoggingBuildStep):
             revision = self.computeSourceRevision(s.changes)
         # if patch is None, then do not patch the tree after checkout
 
-        # 'patch' is None or a tuple of (patchlevel, diff)
+        # 'patch' is None or a tuple of (patchlevel, diff, root)
+        # root is optional.
         patch = s.patch
         if patch:
             self.addCompleteLog("patch", patch[1])
 
+        if self.alwaysUseLatest:
+            revision = None
         self.startVC(branch, revision, patch)
 
     def commandComplete(self, cmd):
@@ -268,7 +274,7 @@ class CVS(Source):
                                read-only (I assume this means it won't
                                use locks to insure atomic access to the
                                ,v files).
-                               
+
         @type  checkout_options: list of strings
         @param checkout_options: these arguments are inserted in the cvs
                                command line, after 'checkout' but before
@@ -361,7 +367,8 @@ class SVN(Source):
 
     def __init__(self, svnurl=None, baseURL=None, defaultBranch=None,
                  directory=None, username=None, password=None,
-                 extra_args=None, **kwargs):
+                 extra_args=None, keep_on_purge=None, ignore_ignores=None,
+                 always_purge=None, depth=None, **kwargs):
         """
         @type  svnurl: string
         @param svnurl: the URL which points to the Subversion server,
@@ -398,6 +405,10 @@ class SVN(Source):
         self.username = username
         self.password = password
         self.extra_args = extra_args
+        self.keep_on_purge = keep_on_purge
+        self.ignore_ignores = ignore_ignores
+        self.always_purge = always_purge
+	self.depth = depth
 
         Source.__init__(self, **kwargs)
         self.addFactoryArguments(svnurl=svnurl,
@@ -407,6 +418,10 @@ class SVN(Source):
                                  username=username,
                                  password=password,
                                  extra_args=extra_args,
+                                 keep_on_purge=keep_on_purge,
+                                 ignore_ignores=ignore_ignores,
+                                 always_purge=always_purge,
+				 depth=depth,
                                  )
 
         if not svnurl and not baseURL:
@@ -478,6 +493,17 @@ class SVN(Source):
         self.args['revision'] = revision
         self.args['patch'] = patch
 
+	#Set up depth if specified
+	if self.depth is not None:
+		if self.slaveVersionIsOlderThan("svn","2.9"):
+			m = ("This buildslave (%s) does not support svn depth "
+			     "arguments.  "
+			     "Refusing to build. "
+			     "Please upgrade the buildslave." % (self.build.slavename))
+                	raise BuildSlaveTooOldError(m)
+		else: 
+			self.args['depth'] = self.depth
+
         if self.username is not None or self.password is not None:
             if self.slaveVersionIsOlderThan("svn", "2.8"):
                 m = ("This buildslave (%s) does not support svn usernames "
@@ -605,6 +631,7 @@ class Git(Source):
     def __init__(self, repourl,
                  branch="master",
                  submodules=False,
+                 ignore_ignores=None,
                  **kwargs):
         """
         @type  repourl: string
@@ -624,10 +651,12 @@ class Git(Source):
         self.addFactoryArguments(repourl=repourl,
                                  branch=branch,
                                  submodules=submodules,
+                                 ignore_ignores=ignore_ignores,
                                  )
         self.args.update({'repourl': repourl,
                           'branch': branch,
-                          'submodules' : submodules,
+                          'submodules': submodules,
+                          'ignore_ignores': ignore_ignores,
                           })
 
     def computeSourceRevision(self, changes):
@@ -1155,4 +1184,3 @@ class Monotone(Source):
         assert slavever, "slave is too old, does not know about monotone"
         cmd = LoggedRemoteCommand("monotone", self.args)
         self.startCommand(cmd)
-
