@@ -18,6 +18,7 @@ config_1 = """
 from buildbot.process import factory
 from buildbot.steps import dummy
 from buildbot.buildslave import BuildSlave
+from buildbot.config import BuilderConfig
 s = factory.s
 
 BuildmasterConfig = c = {}
@@ -33,27 +34,23 @@ f3 = factory.BuildFactory([s(dummy.RemoteDummy, timeout=3)])
 f4 = factory.BuildFactory([s(dummy.RemoteDummy, timeout=5)])
 
 c['builders'] = [
-    {'name': 'b1', 'slavenames': ['bot1','bot2','bot3'],
-     'builddir': 'b1', 'factory': f1},
-    ]
+    BuilderConfig(name='b1', slavenames=['bot1', 'bot2', 'bot3'], factory=f1),
+]
 """
 
 config_2 = config_1 + """
 
 c['builders'] = [
-    {'name': 'b1', 'slavenames': ['bot1','bot2','bot3'],
-     'builddir': 'b1', 'factory': f2},
-    ]
+    BuilderConfig(name='b1', slavenames=['bot1', 'bot2', 'bot3'], factory=f2),
+]
 
 """
 
 config_busyness = config_1 + """
 c['builders'] = [
-    {'name': 'b1', 'slavenames': ['bot1'],
-     'builddir': 'b1', 'factory': f3},
-    {'name': 'b2', 'slavenames': ['bot1'],
-     'builddir': 'b2', 'factory': f4},
-    ]
+    BuilderConfig(name='b1', slavenames=['bot1', ], factory=f3),
+    BuilderConfig(name='b2', slavenames=['bot1', ], factory=f4),
+]
 """
 
 class Slave(RunMixin, unittest.TestCase):
@@ -146,11 +143,11 @@ class Slave(RunMixin, unittest.TestCase):
 
         b1 = self.master.botmaster.builders["b1"]
         # reduce the ping time so we'll failover faster
-        b1.START_BUILD_TIMEOUT = 1
         assert b1.CHOOSE_SLAVES_RANDOMLY
         b1.CHOOSE_SLAVES_RANDOMLY = False
         self.disappearSlave("bot1", "b1", allowReconnect=False)
         d = self.doBuild("b1")
+        self.killSlave("bot1", "b1")
         d.addCallback(self._testFallback2_1)
         return d
     def _testFallback2_1(self, res):
@@ -268,6 +265,7 @@ from buildbot.process import factory
 from buildbot.steps import dummy
 from buildbot.buildslave import BuildSlave
 from buildbot.test.test_slaves import FakeLatentBuildSlave
+from buildbot.config import BuilderConfig
 s = factory.s
 
 BuildmasterConfig = c = {}
@@ -286,9 +284,8 @@ f3 = factory.BuildFactory([s(dummy.RemoteDummy, timeout=3)])
 f4 = factory.BuildFactory([s(dummy.RemoteDummy, timeout=5)])
 
 c['builders'] = [
-    {'name': 'b1', 'slavenames': ['bot1','bot2','bot3'],
-     'builddir': 'b1', 'factory': f1},
-    ]
+    BuilderConfig(name='b1', slavenames=['bot1', 'bot2', 'bot3'], factory=f1),
+]
 """
 
 
@@ -319,7 +316,9 @@ class LatentSlave(RunMixin, unittest.TestCase):
         # make sure both slaves appear in the builder.  This is automatically,
         # without any attaching.
         self.assertEqual(len(self.b1.slaves), 2)
-        self.assertEqual(sorted(sb.slave.slavename for sb in self.b1.slaves),
+        slavenames = [sb.slave.slavename for sb in self.b1.slaves]
+        slavenames.sort()
+        self.assertEqual(slavenames,
                          ['bot1', 'bot2'])
         # These have not substantiated
         self.assertEqual([sb.slave.substantiated for sb in self.b1.slaves],
@@ -334,7 +333,9 @@ class LatentSlave(RunMixin, unittest.TestCase):
     def _testSequence_1(self, res):
         # now we have all three slaves.  Two are latent slaves, and one is a
         # standard slave.
-        self.assertEqual(sorted(sb.slave.slavename for sb in self.b1.slaves),
+        slavenames = [sb.slave.slavename for sb in self.b1.slaves]
+        slavenames.sort()
+        self.assertEqual(slavenames,
                          ['bot1', 'bot2', 'bot3'])
         # Now it's time to try a build on one of the latent slaves,
         # substantiating it.
@@ -552,7 +553,9 @@ class LatentSlave(RunMixin, unittest.TestCase):
         d.addCallback(self._testPing_1)
         return d
     def _testPing_1(self, res):
-        self.assertEqual(sorted(sb.slave.slavename for sb in self.b1.slaves),
+        slavenames = [sb.slave.slavename for sb in self.b1.slaves]
+        slavenames.sort()
+        self.assertEqual(slavenames,
                          ['bot1', 'bot2', 'bot3'])
         d = self.control.getBuilder('b1').ping()
         d.addCallback(self._testPing_2)
@@ -648,6 +651,7 @@ config_3 = """
 from buildbot.process import factory
 from buildbot.steps import dummy
 from buildbot.buildslave import BuildSlave
+from buildbot.config import BuilderConfig
 s = factory.s
 
 BuildmasterConfig = c = {}
@@ -661,23 +665,20 @@ f2 = factory.BuildFactory([s(dummy.Wait, handle='two')])
 f3 = factory.BuildFactory([s(dummy.Wait, handle='three')])
 
 c['builders'] = [
-    {'name': 'b1', 'slavenames': ['bot1'],
-     'builddir': 'b1', 'factory': f1},
-    ]
+    BuilderConfig(name='b1', slavename='bot1', factory=f1),
+]
 """
 
 config_4 = config_3 + """
 c['builders'] = [
-    {'name': 'b1', 'slavenames': ['bot1'],
-     'builddir': 'b1', 'factory': f2},
-    ]
+    BuilderConfig(name='b1', slavename='bot1', factory=f2),
+]
 """
 
 config_5 = config_3 + """
 c['builders'] = [
-    {'name': 'b1', 'slavenames': ['bot1'],
-     'builddir': 'b1', 'factory': f3},
-    ]
+    BuilderConfig(name='b1', slavename='bot1', factory=f3),
+]
 """
 
 from buildbot.slave.commands import waitCommandRegistry
@@ -899,23 +900,21 @@ class Slave2(RunMixin, unittest.TestCase):
 
 config_multi_builders = config_1 + """
 c['builders'] = [
-    {'name': 'dummy', 'slavenames': ['bot1','bot2','bot3'],
-     'builddir': 'b1', 'factory': f2},
-    {'name': 'dummy2', 'slavenames': ['bot1','bot2','bot3'],
-     'builddir': 'b2', 'factory': f2},
-    {'name': 'dummy3', 'slavenames': ['bot1','bot2','bot3'],
-     'builddir': 'b3', 'factory': f2},
-    ]
-
+    BuilderConfig(name='dummy', factory=f2,
+                  slavenames=['bot1', 'bot2', 'bot3']),
+    BuilderConfig(name='dummy2', factory=f2,
+                  slavenames=['bot1', 'bot2', 'bot3']),
+    BuilderConfig(name='dummy3', factory=f2,
+                  slavenames=['bot1', 'bot2', 'bot3']),
+]
 """
 
 config_mail_missing = config_1 + """
 c['slaves'] = [BuildSlave('bot1', 'sekrit', notify_on_missing='admin',
                           missing_timeout=1)]
 c['builders'] = [
-    {'name': 'dummy', 'slavenames': ['bot1'],
-     'builddir': 'b1', 'factory': f1},
-    ]
+    BuilderConfig(name='dummy', slavename='bot1', factory=f1),
+]
 c['projectName'] = 'myproject'
 c['projectURL'] = 'myURL'
 """
