@@ -14,6 +14,7 @@
 # todo: test batched updates, by invoking remote_update(updates) instead of
 # statusUpdate(update). Also involves interrupted builds.
 
+import sys
 import os
 
 from twisted.trial import unittest
@@ -21,10 +22,11 @@ from twisted.internet import reactor, defer
 
 from buildbot.sourcestamp import SourceStamp
 from buildbot.process import buildstep, base, factory
+from buildbot.process.properties import Properties, WithProperties
 from buildbot.buildslave import BuildSlave
 from buildbot.steps import shell, source, python, master
 from buildbot.status import builder
-from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE
+from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, SKIPPED
 from buildbot.test.runutils import RunMixin, rmtree
 from buildbot.test.runutils import makeBuildStep, StepTester
 from buildbot.slave import commands, registry
@@ -129,10 +131,12 @@ class BuildStep(unittest.TestCase):
                                "shell",
                                 {'command': "argle bargle",
                                  'workdir': "murkle",
+                                 'logEnviron' : True,
                                  'want_stdout': 1,
                                  'want_stderr': 1,
                                  'logfiles': {},
                                  'timeout': 10,
+                                 'maxTime': None,
                                  'usePTY': 'slave-config',
                                  'env': None}) ] )
         self.assertEqual(self.remote.events, self.expectedEvents)
@@ -345,14 +349,17 @@ version_config = """
 from buildbot.process import factory
 from buildbot.test.test_steps import VersionCheckingStep
 from buildbot.buildslave import BuildSlave
+from buildbot.config import BuilderConfig
 BuildmasterConfig = c = {}
 f1 = factory.BuildFactory([
     factory.s(VersionCheckingStep),
     ])
 c['slaves'] = [BuildSlave('bot1', 'sekrit')]
 c['schedulers'] = []
-c['builders'] = [{'name':'quick', 'slavename':'bot1',
-                  'builddir': 'quickdir', 'factory': f1}]
+c['builders'] = [
+    BuilderConfig(name='quick', slavename='bot1', factory=f1,
+            builddir='quickdir', slavebuilddir='quickslavedir'),
+]
 c['slavePortnum'] = 0
 """
 
@@ -621,6 +628,112 @@ ending line
         results = step.evaluateCommand(cmd)
         self.failUnlessEqual(results, WARNINGS)
 
+    def testCompile4(self):
+        # Test suppression of warnings.
+        self.masterbase = "Warnings.testCompile4"
+        step = self.makeStep(shell.Compile,
+                             warningPattern="^(.*?):([0-9]+): [Ww]arning: (.*)$",
+                             warningExtractor=shell.Compile.warnExtractFromRegexpGroups,
+                             directoryEnterPattern="make.*: Entering directory [\"`'](.*)['`\"]",
+                             directoryLeavePattern="make.*: Leaving directory")
+        step.addSuppression([(r"/subdir/", r"xyzzy", None, None),
+                             (r"foo.c", r".*", None, 20),
+                             (r"foo.c", r".*", 200, None),
+                             (r"foo.c", r".*", 50, 50),
+                             (r"xxx", r".*", None, None),
+                             ])
+        log = step.addLog("stdio")
+        output = \
+"""Making all in .
+make[1]: Entering directory `/abs/path/build'
+foo.c:10: warning: `bar' defined but not used
+foo.c:50: warning: `bar' defined but not used
+make[2]: Entering directory `/abs/path/build/subdir'
+baz.c:33: warning: `xyzzy' defined but not used
+baz.c:34: warning: `magic' defined but not used
+make[2]: Leaving directory `/abs/path/build/subdir'
+foo.c:100: warning: `xyzzy' defined but not used
+foo.c:200: warning: `bar' defined but not used
+make[2]: Leaving directory `/abs/path/build'
+"""
+        log.addStdout(output)
+        log.finish()
+        step.createSummary(log)
+        self.failUnlessEqual(step.getProperty("warnings-count"), 2)
+        logs = {}
+        for log in step.step_status.getLogs():
+            logs[log.getName()] = log
+        self.failUnless("warnings" in logs)
+        lines = logs["warnings"].readlines()
+        self.failUnlessEqual(len(lines), 2)
+        self.failUnlessEqual(lines[0], "baz.c:34: warning: `magic' defined but not used\n")
+        self.failUnlessEqual(lines[1], "foo.c:100: warning: `xyzzy' defined but not used\n")
+
+        cmd = buildstep.RemoteCommand(None, {})
+        cmd.rc = 0
+        results = step.evaluateCommand(cmd)
+        self.failUnlessEqual(results, WARNINGS)
+
+    def filterArgs(self, args):
+        if "writer" in args:
+            args["writer"] = self.wrap(args["writer"])
+        return args
+
+    suppressionFileData = """
+# Sample suppressions file for testing
+
+/subdir/ : xyzzy
+foo.c: .* : 0-20
+foo.c: .*: 200-10000
+foo.c :.*: 50
+xxx : .*
+"""
+    def testCompile5(self):
+        # Test downloading warning suppression file from slave.
+        self.slavebase = "Warnings.testCompile5.slave"
+        self.masterbase = "Warnings.testCompile5.master"
+        sb = self.makeSlaveBuilder()
+        os.mkdir(os.path.join(self.slavebase, self.slavebuilderbase,
+                              "build"))
+        output = \
+"""Making all in .
+make[1]: Entering directory `/abs/path/build'
+foo.c:10: warning: `bar' defined but not used
+foo.c:50: warning: `bar' defined but not used
+make[2]: Entering directory `/abs/path/build/subdir'
+baz.c:33: warning: `xyzzy' defined but not used
+baz.c:34: warning: `magic' defined but not used
+make[2]: Leaving directory `/abs/path/build/subdir'
+foo.c:100: warning: `xyzzy' defined but not used
+foo.c:200: warning: `bar' defined but not used
+make[2]: Leaving directory `/abs/path/build'
+"""
+        printStatement = ('print """%s"""' % output)
+        step = self.makeStep(shell.Compile,
+                             warningPattern="^(.*?):([0-9]+): [Ww]arning: (.*)$",
+                             warningExtractor=shell.Compile.warnExtractFromRegexpGroups,
+                             suppressionFile="warnings.supp",
+                             command=[sys.executable, "-c", printStatement])
+        slavesrc = os.path.join(self.slavebase,
+                                self.slavebuilderbase,
+                                "build",
+                                "warnings.supp")
+        open(slavesrc, "w").write(self.suppressionFileData)
+
+        d = self.runStep(step)
+        def _checkResult(result):
+            self.failUnlessEqual(step.getProperty("warnings-count"), 2)
+            logs = {}
+            for log in step.step_status.getLogs():
+                logs[log.getName()] = log
+            self.failUnless("warnings" in logs)
+            lines = logs["warnings"].readlines()
+            self.failUnlessEqual(len(lines), 2)
+            self.failUnlessEqual(lines[0], "baz.c:34: warning: `magic' defined but not used\n")
+            self.failUnlessEqual(lines[1], "foo.c:100: warning: `xyzzy' defined but not used\n")
+
+        d.addCallback(_checkResult)
+        return d
 
 class TreeSize(StepTester, unittest.TestCase):
     def testTreeSize(self):
@@ -751,7 +864,9 @@ class MasterShellCommand(StepTester, unittest.TestCase):
         self.slavebase = "testMasterShellCommand.slave"
         self.masterbase = "testMasterShellCommand.master"
         sb = self.makeSlaveBuilder()
-        step = self.makeStep(master.MasterShellCommand, command=['echo', 'hi'])
+        step = self.makeStep(master.MasterShellCommand, command=['echo',
+                                   WithProperties("hi build-%(other)s.tar.gz")])
+        step.build.setProperty("other", "foo", "test")
 
         # we can't invoke runStep until the reactor is started .. hence this
         # little dance
@@ -763,7 +878,7 @@ class MasterShellCommand(StepTester, unittest.TestCase):
         def _check(results):
             self.failUnlessEqual(results, SUCCESS)
             logtxt = step.getLog("stdio").getText()
-            self.failUnlessEqual(logtxt.strip(), "hi")
+            self.failUnlessEqual(logtxt.strip(), "hi build-foo.tar.gz")
         d.addCallback(_check)
         reactor.callLater(0, d.callback, None)
         return d
@@ -786,3 +901,52 @@ class MasterShellCommand(StepTester, unittest.TestCase):
         d.addCallback(_check)
         reactor.callLater(0, d.callback, None)
         return d
+
+class SuccessStep(buildstep.BuildStep):
+    def start(self):
+        self.finished(buildstep.SUCCESS)
+
+class ConditionalStepTest(StepTester, unittest.TestCase):
+    def testNotSkipped(self):
+        self.slavebase = "testNotSkipped.slave"
+        self.masterbase = "testNotSkipped.master"
+        sb = self.makeSlaveBuilder()
+        step = self.makeStep(SuccessStep)
+        d = self.runStep(step)
+        def _checkResults(results):
+            self.failUnlessEqual(SUCCESS, results)
+        d.addCallback(_checkResults)
+        return d
+
+    def testSkipped(self):
+        self.slavebase = "testSkipped.slave"
+        self.masterbase = "testSkipped.master"
+        sb = self.makeSlaveBuilder()
+        step = self.makeStep(SuccessStep, doStepIf=False)
+        d = self.runStep(step)
+        def _checkResults(results):
+            self.failUnlessEqual(SKIPPED, results)
+        d.addCallback(_checkResults)
+        return d
+
+    def testNotSkippedFunc(self):
+        self.slavebase = "testNotSkippedFunc.slave"
+        self.masterbase = "testNotSkippedFunc.master"
+        sb = self.makeSlaveBuilder()
+        step = self.makeStep(SuccessStep, doStepIf=lambda s: True)
+        d = self.runStep(step)
+        def _checkResults(results):
+            self.failUnlessEqual(SUCCESS, results)
+        d.addCallback(_checkResults)
+        return d
+
+    def testSkippedFunc(self):
+        self.slavebase = "testSkippedFunc.slave"
+        self.masterbase = "testSkippedFunc.master"
+        sb = self.makeSlaveBuilder()
+        step = self.makeStep(SuccessStep, doStepIf=lambda s: False)
+        d = self.runStep(step)
+        def _checkResults(results):
+            self.failUnlessEqual(SKIPPED, results)
+        d.addCallback(_checkResults)
+        return d
