@@ -1,4 +1,5 @@
 # -*- test-case-name: buildbot.test.test_web -*-
+# -*- coding: utf-8 -*-
 
 import os, time, shutil
 import warnings
@@ -14,7 +15,7 @@ from twisted.web import client
 
 from buildbot import master, interfaces, sourcestamp
 from buildbot.status import html, builder
-from buildbot.status.web import waterfall
+from buildbot.status.web import waterfall, xmlrpc
 from buildbot.changes.changes import Change
 from buildbot.process import base
 from buildbot.process.buildstep import BuildStep
@@ -41,15 +42,17 @@ from buildbot.status import html
 from buildbot.buildslave import BuildSlave
 from buildbot.scheduler import Scheduler
 from buildbot.process.factory import BuildFactory
+from buildbot.config import BuilderConfig
 
 BuildmasterConfig = c = {
     'change_source': PBChangeSource(),
     'slaves': [BuildSlave('bot1name', 'bot1passwd')],
     'schedulers': [Scheduler('name', None, 60, ['builder1'])],
-    'builders': [{'name': 'builder1', 'slavename': 'bot1name',
-                  'builddir': 'builder1', 'factory': BuildFactory()}],
     'slavePortnum': 0,
     }
+c['builders'] = [
+    BuilderConfig(name='builder1', slavename='bot1name', factory=BuildFactory()),
+]
 """
 
 
@@ -311,6 +314,7 @@ from buildbot.steps import dummy
 from buildbot.scheduler import Scheduler
 from buildbot.changes.base import ChangeSource
 from buildbot.buildslave import BuildSlave
+from buildbot.config import BuilderConfig
 s = factory.s
 
 class DiscardScheduler(Scheduler):
@@ -329,9 +333,8 @@ c['status'] = [html.Waterfall(http_port=0)]
 f = factory.BuildFactory([s(dummy.RemoteDummy, timeout=1)])
 
 c['builders'] = [
-    {'name': 'b1', 'slavenames': ['bot1','bot2'],
-     'builddir': 'b1', 'factory': f},
-    ]
+    BuilderConfig(name='b1', slavenames=['bot1', 'bot2'], factory=f),
+]
 c['buildbotURL'] = 'http://dummy.example.org:8010/'
 
 """
@@ -417,15 +420,18 @@ class Logfile(BaseWeb, RunMixin, unittest.TestCase):
 from buildbot.status import html
 from buildbot.process.factory import BasicBuildFactory
 from buildbot.buildslave import BuildSlave
+from buildbot.config import BuilderConfig
+
 f1 = BasicBuildFactory('cvsroot', 'cvsmodule')
-BuildmasterConfig = {
+BuildmasterConfig = c = {
     'slaves': [BuildSlave('bot1', 'passwd1')],
     'schedulers': [],
-    'builders': [{'name': 'builder1', 'slavename': 'bot1',
-                  'builddir':'workdir', 'factory':f1}],
     'slavePortnum': 0,
     'status': [html.WebStatus(http_port=0)],
     }
+c['builders'] = [
+    BuilderConfig(name='builder1', slavename='bot1', factory=f1),
+]
 """
         if os.path.exists("test_logfile"):
             shutil.rmtree("test_logfile")
@@ -450,7 +456,7 @@ BuildmasterConfig = {
         bss.stepStarted()
 
         log1 = step1.addLog("output")
-        log1.addStdout("some stdout\n")
+        log1.addStdout(u"sÒme stdout\n")
         log1.finish()
 
         log2 = step1.addHTMLLog("error", "<html>ouch</html>")
@@ -504,7 +510,8 @@ BuildmasterConfig = {
         logurl = self.getLogURL("setup", "output")
         d = client.getPage(logurl + "/text")
         def _check(logtext):
-            self.failUnlessEqual(logtext, "some stdout\n")
+            # verify utf-8 encoding.
+            self.failUnlessEqual(logtext, "sÒme stdout\n")
         d.addCallback(_check)
         return d
 
@@ -601,3 +608,13 @@ BuildmasterConfig = {
         d.addCallback(_check)
         d.addErrback(_fail)
         return d
+
+class XMLRPC(unittest.TestCase):
+    def test_init(self):
+        server = xmlrpc.XMLRPCServer()
+        self.assert_(server)
+
+    def test_render(self):
+        self.assertRaises(NameError,
+                          lambda:
+                              xmlrpc.XMLRPCServer().render(Request()))
